#
# $Id: Rules.make,v 1.1.1.1 1998/07/10 14:16:35 joel Exp $
#
# This file contains rules which are shared between multiple Makefiles.
#
# Adapted from the Linux kernel by Plessey Asia Pacific.
#

#
# False targets.
#
.PHONY: dummy

#
# Special variables which should not be exported
#
unexport EXTRA_ASFLAGS
unexport EXTRA_CFLAGS
unexport EXTRA_LDFLAGS
unexport EXTRA_ARFLAGS
unexport SUBDIRS
unexport SUB_DIRS
unexport ALL_SUB_DIRS
unexport MOD_SUB_DIRS
unexport D_TARGET
unexport L_TARGET
unexport O_TARGET
unexport O_OBJS
unexport L_OBJS
unexport M_OBJS
unexport ALL_MOBJS
unexport SUB_O
unexport SUB_LL
unexport ALL_O
unexport ALL_L
# objects that export symbol tables
unexport OX_OBJS
unexport LX_OBJS
unexport MX_OBJS
unexport SYMTAB_OBJS

unexport MOD_LIST_NAME

#
# Generate the output file names for each directory name if not already done and a directory
# name is defined
#
ifdef D_TARGET
ifndef O_TARGET
O_TARGET = $(notdir $(D_TARGET).io)
endif

ifndef L_TARGET
L_TARGET = $(notdir $(D_TARGET).a)
endif
endif

#
# Set the VPATH to the output directory so all the rules work
#

VPATH = $(OUTPUT_PATH)

#
# A rule to make subdirectories
#
ifdef SUB_DIRS
all_targets: sub_dirs check_output_path $(O_TARGET) $(L_TARGET)

# defined the directory target so we know to link
sub_dirs: dummy
	set -e; for i in $(SUB_DIRS); \
	  do $(MAKE) -C $$i D_TARGET=$$i OUTPUT_PATH=$(OUTPUT_PATH)/$$i; \
	done
else
all_targets: check_output_path $(O_TARGET) $(L_TARGET)
endif

#
# Check to see if the output path is present
#

check_output_path:
	if [ ! -d "$(OUTPUT_PATH)" ]; then \
	  mkdir -p $(OUTPUT_PATH); fi

#
# Common rules
#

$(OUTPUT_PATH)/%.s: %.c
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -S -o $(OUTPUT_PATH)/$(@F) $<

%.s: %.c
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -S -o $(OUTPUT_PATH)/$(@F) $<

$(OUTPUT_PATH)/%.o: %.cc
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

%.o: %.cc
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

$(OUTPUT_PATH)/%.o: %.cpp
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

%.o: %.cpp
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

$(OUTPUT_PATH)/%.o: %.c
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

%.o: %.c
	$(CC) $(CFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

$(OUTPUT_PATH)/%.o: %.S
	$(CC) $(ASFLAGS) $(EXTRA_CFLAGS) -E -o $(OUTPUT_PATH)/$(patsubst %.o,%.s,$(@F)) $<
	$(CC) $(ASFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $(OUTPUT_PATH)/$(patsubst %.o,%.s,$(@F))

%.o: %.S
	$(CC) $(ASFLAGS) $(EXTRA_CFLAGS) -E -o $(OUTPUT_PATH)/$(patsubst %.o,%.s,$(@F)) $<
	$(CC) $(ASFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $(OUTPUT_PATH)/$(patsubst %.o,%.s,$(@F))

$(OUTPUT_PATH)/%.o: %.s
	$(AS) $(ASFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

%.o: %.s
	$(AS) $(ASFLAGS) $(EXTRA_CFLAGS) -c -o $(OUTPUT_PATH)/$(@F) $<

#
# Rule to compile a set of .o files into one .o file
#
ifdef O_TARGET
ifneq "$(strip $(SUB_DIRS))" ""
SUB_O = $(foreach A_DIR, $(strip $(SUB_DIRS)), $(A_DIR)/$(A_DIR).io)
endif
ALL_O = $(addprefix $(OUTPUT_PATH)/, $(OX_OBJS) $(O_OBJS) $(SUB_O))
$(O_TARGET): $(ALL_O) 
	rm -f $(OUTPUT_PATH)/$(@F)
ifneq "$(strip $(ALL_O))" ""
	$(LD) $(EXTRA_LDFLAGS) -r -o $(OUTPUT_PATH)/$(@F) $^
else
	$(AR) rcs $(OUTPUT_PATH)/$(@F)
endif
endif

#
# Rule to compile a set of .o files into one .a file
#
# The objects of the sub directories are loaded into .a files of the current
# directory. The object list for this directory is found in the $(L_TARGET_TABLE).
# This is the table of contents of the library. It is formed by merging in the sub
# directory's table of contents files and the current directory.
#
ifdef L_TARGET
ifneq "$(strip $(SUB_DIRS))" ""
SUB_LL = $(foreach A_DIR, $(strip $(SUB_DIRS)), $(OUTPUT_PATH)/$(A_DIR)/$(A_DIR).a)
endif
L_TARGET_TABLE = $(basename $(@F)).ar
ALL_L = $(LX_OBJS) $(L_OBJS)
$(L_TARGET): $(addprefix $(OUTPUT_PATH)/, $(ALL_L)) $(SUB_LL)
	rm -f $(OUTPUT_PATH)/$(L_TARGET_TABLE)
ifneq "$(ALL_L)" ""
	echo $(addprefix $(OUTPUT_PATH)/, $(ALL_L)) > $(OUTPUT_PATH)/$(L_TARGET_TABLE)
endif
ifneq "$(SUB_DIRS)" ""
	set -e; for i in $(SUB_DIRS); \
	  do if [ -f $(OUTPUT_PATH)/$$i/$$i.ar ]; then \
	    cat $(OUTPUT_PATH)/$$i/$$i.ar >> $(OUTPUT_PATH)/$(L_TARGET_TABLE); fi \
	  done
endif
	$(AR) $(EXTRA_ARFLAGS) rcs $(OUTPUT_PATH)/$(@F) `cat $(OUTPUT_PATH)/$(L_TARGET_TABLE)`
endif

#
# A rule to do nothing
#
dummy:

#
# This is useful for testing
#
script:
	$(SCRIPT)


